<?php
if (!isset($sshfed_folder)) {
	require_once("../../config.php");
}
require_once($sshfed_folder."/api/Backend.php");
require_once($sshfed_folder."/api/SSHServer.php");
require_once($siledap_folder."/LdapConf.php");
require_once($siledap_folder."/LdapManager.php");
require_once($siledap_folder."/LdapEntry.php");

/*
		<Backend value="ldap">
			<Server host="localhost" port="389" />
			<BaseDN value="dc=groups,dc=urn,dc=myserver,dc=org" />
			<BindDN value="uid=admin,dc=urn,dc=myserver,dc=org" password="ldap.secure.666" />
		</Backend>
*/
class BackendLdap implements Backend {
	private $idGroup;
	private $server;
	private $serverPort;
	private $basedn;
	private $binddn;
	private $bindpass;
	
	public function __construct($idGroup, $xmlElem = "") {
		$this->idGroup = $idGroup;
		$this->server = "";
		$this->serverPort = 389;
		$this->basedn = "";
		$this->binddn = "";
		$this->bindpass = "";
		
		if (!is_string($xmlElem)) {
			$this->loadFromXML($xmlElem);
		}
	}
	
	private function loadFromXML($xmlElem) {
		$this->server = (string) $xmlElem->Server["host"];
		$this->serverPort = (int) $xmlElem->Server["port"];
		$this->basedn = (string) $xmlElem->BaseDN["value"];
		$this->binddn = (string) $xmlElem->BindDN["value"];
		$this->bindpass = (string) $xmlElem->BindDN["password"];
	}
	
	public function getIDGroup() {
		return $this->idGroup;
	}
	
	public function setServerHost($host) {
		$this->server = $host;
	}
	
	public function getServerHost() {
		return $this->server;
	}
	
	public function setServerPort($port) {
		$this->serverPort = $port;
	}
	
	public function getServerPort() {
		return $this->serverPort;
	}
	
	public function setBaseDN($dn) {
		$this->basedn = $dn;
	}
	
	public function getBaseDN() {
		return $this->basedn;
	}
	
	public function setBindUser($user) {
		$this->binddn = $user;
	}
	
	public function getBindUser() {
		return $this->binddn;
	}
	
	public function setBindPassword($pass) {
		$this->bindpass = $pass;
	}
	
	public function getBindPassword() {
		return $this->bindpass;
	}
	
	public function getServers() {
		$res = array();
		
		$lm=new LdapManager($this->getServerHost(),$this->getServerPort());
		$lm->setOption(LDAP_OPT_PROTOCOL_VERSION,3);
		$lm->connectLdap($this->getBindUser(),$this->getBindPassword());
		
		$filter = "(cn=".$this->getIDGroup().")";
		error_log("Search ".$filter." on ".$this->getBaseDN());
		$resLdap=$lm->search($this->getBaseDN(),$filter,true);
		if (count($resLdap)<1) {
			throw new Exception("There is no entry for the group ".$this->getIDGroup());
		}
		else if (count($resLdap)>1) {
			throw new Exception("Too many entries for the group ".$this->getIDGroup());
		}
		else {
			$rootEntry = $resLdap[0];
			$filter = "(idnc=*)";
			error_log("Search ".$filter." on ".$rootEntry->getDN());
			$resLdap2=$lm->search($rootEntry->getDN(),$filter,true);
			foreach ($resLdap2 as $entry) {
				$res[] = $this->getFromLDAPEntry($entry);
			}
		}
		
		return $res;
	}
	
	private function getFromLDAPEntry($entry) {
		$res = new SSHServer();
		
		$address = $entry->getSingleAttribute("ipHostNumber");
		$addressTemp = explode(":",$address);
		$res->setHost($addressTemp[0]);
		$res->setPort($addressTemp[1]);
		
		$res->setCheckMode("urn:mace:rediris.es:check:none");
		
		return $res;
	}
	
	public function initGroup() {
		$lm=new LdapManager($this->getServerHost(),$this->getServerPort());
		$lm->setOption(LDAP_OPT_PROTOCOL_VERSION,3);
		$lm->connectLdap($this->getBindUser(),$this->getBindPassword());

		$filter = "(cn=".$this->getIDGroup().")";
		error_log("Search ".$filter." on ".$this->getBaseDN());
		$resLdap=$lm->search($this->getBaseDN(),$filter,false);
		if (count($resLdap)>1) {
			throw new Exception("Something is wrong");
		}
		else if (count($resLdap)==1) {
			$entry = $resLdap[0];
			$idEntry = $entry->getSingleAttribute("cn");
			if (strcmp($idEntry,$this->getIDGroup())!=0) {
				throw new Exception("Entry at base dn is not free");
			}
		}
		else {
			$idnc = $this->uuid();
			$entry = new LdapEntry("idnc=".$idnc.",".$this->getBaseDN());
			$entry->addAttribute("objectClass","irisInetEntityStr");
			$entry->addAttribute("objectClass","irisObject");
			$entry->addAttribute("idnc",$idnc);
			$entry->addAttribute("cn",$this->getIDGroup());
	
			$objClassList=$entry->getObjectClassList()->getValues();
			error_log(print_r($objClassList,true));
			$attrList=$entry->getAttributeValues();
			error_log(print_r($attrList,true));
			if ($lm->addEntry($entry->getDN(),$objClassList,$attrList)==false) {
				throw new Exception("Cannot add the initial entry");
			}
			
		}
	}
	
	private function uuid() {
		return sprintf( '%04x%04x-%04x-%04x-%04x-%04x%04x%04x',
			mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff ),
			mt_rand( 0, 0x0fff ) | 0x4000,
			mt_rand( 0, 0x3fff ) | 0x8000,
			mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff ) );
	}
}
?>