<?php
require_once("../../config.php");
require_once($sshfed_folder."/api/SSHList.php");
require_once($sshfed_folder."/api/SSHServer.php");
require_once($siledap_folder."/LdapConf.php");
require_once($siledap_folder."/LdapManager.php");
require_once($siledap_folder."/LdapEntry.php");

date_default_timezone_set('UTC');

$LIST_SERVERS = SSHList::readFromConfigFile();

function uuid() {
	return sprintf( '%04x%04x-%04x-%04x-%04x-%04x%04x%04x',
		mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff ),
		mt_rand( 0, 0x0fff ) | 0x4000,
		mt_rand( 0, 0x3fff ) | 0x8000,
		mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff ) );
}

function getTimeStamp($exp) {
	$year = substr($exp,0,4);
	$month = substr($exp,4,2);
	$day = substr($exp,6,2);
	$hour = substr($exp,8,2);
	$min = substr($exp,10,2);
	$seg = substr($exp,12,2);
	$res = mktime($hour,$min,$seg,$month,$day,$year);
	
	return $res;
}

function sortByExpiryDate($a, $b) {
	$expA = $a->getSingleAttribute("schacExpiryDate");
	$expB = $b->getSingleAttribute("schacExpiryDate");
	
	return (getTimeStamp($expA) > getTimeStamp($expB)) ? -1 : 1;
}

function getUserId() {
	return "kan";
}

function updateKey($server,$user,$error) {
	GLOBAL $LDAP_SERVER_NAME,$LDAP_SERVER_PORT,$LDAP_USER,$LDAP_PASS,$LDAP_DN_BASE;
	GLOBAL $LIST_SERVERS;

	$userId = getUserId();
	$isGroup = true;
	if (strpos($server,"group@")===FALSE) {
		$isGroup = false;
	}
	$serverData = explode(":",$server);
	$timeout = 0;
	$expiry = 0;
	if ($isGroup == false) {
		foreach ($LIST_SERVERS->getSSHServerList() as $serverElem) {
			if ((strcmp($serverElem->getHost(),$serverData[0])==0)&&($serverElem->getPort()==$serverData[1])) {
				foreach ($serverElem->getPolicies() as $policy) {
					if (strcmp($policy->getUser(),$user)==0) {
						$timeout = $policy->getTimeout();
						$timeout = 60*$timeout;
						$expiry = time() + $timeout;
					}
				}
			}
		}
	}
	else {
		$id = substr($server,6);
		foreach ($LIST_SERVERS->getSSHGroupList() as $group) {
			if (strcmp($group->getID(),$id)==0) {
				foreach ($group->getPolicies() as $policy) {
					if (strcmp($policy->getUser(),$user)==0) {
						$timeout = $policy->getTimeout();
						$timeout = 60*$timeout;
						$expiry = time() + $timeout;
					}
				}
			}
		}
	}
	
	if ($isGroup==false&&strcmp($serverData[0],"")==0) {
		$error = "Wrong host";
	}
	else if ($isGroup==false&&($serverData[1]<1||$serverData[1]>65535)) {
		$error = "Wrong port";
	}
	else if (strcmp($user,"")==0) {
		$error = "Wrong user account";
	}
	else if (strcmp($userId,"")==0) {
		$error = "Cannot get user's id";
	}
	else if ($timeout==0||strcmp($timeout,"")==0) {
		$error = "Bad timeout for the account";
	}
	else {
		$lm=new LdapManager($LDAP_SERVER_NAME,$LDAP_SERVER_PORT);
		$lm->setOption(LDAP_OPT_PROTOCOL_VERSION,3);
		$lm->connectLdap($LDAP_USER,$LDAP_PASS);
		
		$filter = "(&(uid=".$userId.")(schacUserStatus=urn:mace:terena.org:schac:userStatus:es:rediris.es:sshaccess:".urlencode($server)."+user=".$user."))";
		$resLdap=$lm->search($LDAP_DN_BASE,$filter,true);
		if (count($resLdap)<1) {
			$error = "Key not found";
		}
		else {
			usort($resLdap, "sortByExpiryDate");
			for ($i=0; $i<min(count($resLdap),1); $i++) {
				$entry = $resLdap[$i];
				$oldAttrs=$entry->getAttributes();
				$attrSchac = $entry->getAttribute("schacExpiryDate");
				$attrSchac->clearValues();
				$attrSchac->addValue(date("YmdHis",$expiry)."Z","");
				
				$objClassList=$entry->getObjectClassList()->getValues();
				$attrList=$entry->getAttributeValues();
		        if ($lm->replaceEntry($entry->getDN(),array_keys($oldAttrs),$objClassList,$attrList)==false) {
					$error = "Error updating key";
				}
			}
		}
	}		
}

function cancelKey($server,$user,$error) {
	GLOBAL $LDAP_SERVER_NAME,$LDAP_SERVER_PORT,$LDAP_USER,$LDAP_PASS,$LDAP_DN_BASE;
	GLOBAL $LIST_SERVERS;
	
	$res = 0;
	$writen = false;
	
	$userId = getUserId();
	if (strcmp($server,"")==0) {
		$error = "Wrong server";
	}
	else if (strcmp($user,"")==0) {
		$error = "Wrong user account";
	}	
	else if (strcmp($userId,"")==0) {
		$error = "Cannot get user's id";
	}
	else {
		$lm=new LdapManager($LDAP_SERVER_NAME,$LDAP_SERVER_PORT);
		$lm->setOption(LDAP_OPT_PROTOCOL_VERSION,3);
		$lm->connectLdap($LDAP_USER,$LDAP_PASS);
		
		$filter = "(&(uid=".$userId.")(schacUserStatus=urn:mace:terena.org:schac:userStatus:es:rediris.es:sshaccess:".urlencode($server)."+user=".$user."))";
		$resLdap=$lm->search($LDAP_DN_BASE,$filter,true,array("dn"));
		foreach ($resLdap as $entry) {
			$res=$lm->delEntry($entry->getDN(),false);
			if ($res==false) {
				$error = "Error doing the action";
				return;
			}
		}
	}		
}

function checkWS($server, $ws, $error) {
	GLOBAL $LIST_SERVERS;
	
	if (strpos($server,"group@")===FALSE) {
		$serverData = explode(":", $server);
		$theserver = null;
		foreach ($LIST_SERVERS->getSSHServerList() as $serverElem) {
			if ((strcmp($serverElem->getHost(),$serverData[0])==0)&&($serverElem->getPort()==$serverData[1])) {
				$theserver = $serverElem;
			}
		}
		$cron = "";
		$status = true;
		$ch = curl_init();
		if (strpos($theserver->getCheckModeData(),"+")===false) {
			$url = urldecode($theserver->getCheckModeData());
		}
		else {
			$urlData = explode("+",$theserver->getCheckModeData());
			$url = urldecode($urlData[0]);
			$cronTemp = $urlData[1];
			$cronTemp2 = explode("=", $cronTemp);
			$cron = 'cron="'.$cronTemp2[1].'" ';
		}
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_HEADER, 0); 
		if (curl_exec($ch) == false) {
			$error = "Fail";
			$status = false;
		}
		curl_close ($ch);
		echo '<SSHServer host="'.$serverData[0].'" port="'.$serverData[1].'" '.(strcmp($cron,"")!=0?$cron:"").'ws="'.($status==true?"OK":"Fail").'" />';
	}
	else {
		$id = substr($server,6);
		$foundId = false;
		foreach ($LIST_SERVERS->getSSHGroupList() as $group) {
			if (strcmp($group->getID(),$id)==0) {
				$foundId = true;
				$res = "";
				$servers = $group->getServers();
				$good = 0;
				$bad = 0;
				$total = count($servers);
				foreach ($servers as $server) {
					$serverHost = $server->getHost();
					$serverPort = $server->getPort();
					
					$mode = $server->getCheckMode();
					$value = "";
					if (strcmp($mode,"ws")==0) {
						$status = true;
						$serverWs = $server->getCheckModeData();
	
						$ch = curl_init();
						curl_setopt($ch, CURLOPT_URL, $ws);
						curl_setopt($ch, CURLOPT_HEADER, 0); 
						if (curl_exec($ch) == false) {
							$status = false;
						}
						curl_close ($ch);
						
						if ($status==true) {
							$good++;
							$value = "OK";
						}
						else {
							$bad++;
							$value = "Fail";
						}
					}
					else if (strcmp($mode,"cron")==0) {
						$value = $server->getCheckModeData();
					}
					
					$res .= '<SSHServer host="'.$serverHost.'" port="'.$serverPort.'" mode="'.$mode.'" check="'.$value.'" />';
				}
				echo '<SSHGroup id="'.$id.'" total="'.$total.'" good="'.$good.'" bad="'.$bad.'">';
				echo $res;
				echo '</SSHGroup>';
			}
		}
		if ($foundId == false) {
			$error = "Group ID not found";
		}
	}
}

function checkKey($server,$user,$error) {
	GLOBAL $LDAP_SERVER_NAME,$LDAP_SERVER_PORT,$LDAP_USER,$LDAP_PASS,$LDAP_DN_BASE;
	GLOBAL $LIST_SERVERS;
	
	$lm=new LdapManager($LDAP_SERVER_NAME,$LDAP_SERVER_PORT);
	$lm->setOption(LDAP_OPT_PROTOCOL_VERSION,3);
	$lm->connectLdap($LDAP_USER,$LDAP_PASS);
			
	$res = 0;
	if (strpos($server,"group@")===FALSE) {
		$writen = false;
		
		$userId = getUserId();
		$serverData = explode(":",$server);
		$theserver = null;
		foreach ($LIST_SERVERS->getSSHServerList() as $serverElem) {
			if ((strcmp($serverElem->getHost(),$serverData[0])==0)&&($serverElem->getPort()==$serverData[1])) {
				$theserver = $serverElem;
			}
		}
		$host = $serverData[0];
		$port = $serverData[1];
		if (strcmp($host,"")==0) {
			$error = "Wrong host";
		}
		else if ($port<1||$port>65535) {
			$error = "Wrong port";
		}
		else if (strcmp($user,"")==0) {
			$error = "Wrong user account";
		}	
		else {
			$filter = "(&(uid=".$userId.")(schacUserStatus=urn:mace:terena.org:schac:userStatus:es:rediris.es:sshaccess:".urlencode(implode(":",$serverData))."+user=".$user."))";
			$resLdap=$lm->search($LDAP_DN_BASE,$filter,true,array("schacExpiryDate","sshPublicKey","modifyTimestamp"));
			usort($resLdap, "sortByExpiryDate");
			for ($i=0; $i<min(count($resLdap),1); $i++) {
				$entry = $resLdap[$i];
				$exp = $entry->getSingleAttribute("schacExpiryDate");
				$res = getTimeStamp($exp);
				$key = $entry->getSingleAttribute("sshPublicKey");
				
				$keyArray = explode(" ",$key);
				if (strcmp($theserver->getCheckMode(),"cron")==0||strcmp($theserver->getCheckMode(),"ws")==0) {
					echo '<SSHServer check="'.$theserver->getCheckMode().'" checkvalue="'.$theserver->getCheckModeData().'" validFrom="'.getTimeStamp($entry->getSingleAttribute("modifyTimestamp")).'" validUntil="'.$res.'" key="'.$keyArray[1].'" />';
				}
				else {
					echo '<SSHServer validUntil="'.$res.'" key="'.$keyArray[1].'" />';
				}
				$writen = true;
			}
		}		
		if ($writen == false) {
			echo '<SSHServer validUntil="'.$res.'" key="" />';
		}
	}
	else {
		$writen = false;

		$userId = getUserId();
		$id = substr($server,6);
		if (strcmp($id,"")==0) {
			$error = "Wrong group";
		}
		else if (strcmp($user,"")==0) {
			$error = "Wrong user account";
		}	
		else {
			$filter = "(&(uid=".$userId.")(schacUserStatus=urn:mace:terena.org:schac:userStatus:es:rediris.es:sshaccess:".urlencode($server)."+user=".$user."))";
			$resLdap=$lm->search($LDAP_DN_BASE,$filter,true,array("schacExpiryDate","sshPublicKey"));
			usort($resLdap, "sortByExpiryDate");
			for ($i=0; $i<min(count($resLdap),1); $i++) {
				$entry = $resLdap[$i];
				$exp = $entry->getSingleAttribute("schacExpiryDate");
				$res = getTimeStamp($exp);
				$key = $entry->getSingleAttribute("sshPublicKey");
				$keyArray = explode(" ",$key);
				echo '<SSHGroup id="'.$id.'" validUntil="'.$res.'" key="'.$keyArray[1].'" />';
				$writen = true;
			}
		}
		if ($writen == false) {
			echo '<SSHGroup id="'.$id.'" validUntil="'.$res.'" key="" />';
		}
	}
}

function addKey($server,$alg,$key,$user,$error) {
	GLOBAL $LDAP_SERVER_NAME,$LDAP_SERVER_PORT,$LDAP_USER,$LDAP_PASS,$LDAP_DN_BASE;
	GLOBAL $LIST_SERVERS;
	
	$userId = getUserId();
	$isGroup = true;
	if (strpos($server,"group@")===FALSE) {
		$isGroup = false;
	}
	$serverData = explode(":",$server);
	$timeout = 0;
	$expiry = 0;
	if ($isGroup == false) {
		foreach ($LIST_SERVERS->getSSHServerList() as $serverElem) {
			if ((strcmp($serverElem->getHost(),$serverData[0])==0)&&($serverElem->getPort()==$serverData[1])) {
				foreach ($serverElem->getPolicies() as $policy) {
					if (strcmp($policy->getUser(),$user)==0) {
						$timeout = $policy->getTimeout();
						$timeout = 60*$timeout;
						$expiry = time() + $timeout;
					}
				}
			}
		}
	}
	else {
		$id = substr($server,6);
		foreach ($LIST_SERVERS->getSSHGroupList() as $group) {
			if (strcmp($group->getID(),$id)==0) {
				foreach ($group->getPolicies() as $policy) {
					if (strcmp($policy->getUser(),$user)==0) {
						$timeout = $policy->getTimeout();
						$timeout = 60*$timeout;
						$expiry = time() + $timeout;
					}
				}
			}
		}
	}
	
	if ($isGroup==false&&strcmp($serverData[0],"")==0) {
		$error = "Wrong host";
	}
	else if ($isGroup==false&&($serverData[1]<1||$serverData[1]>65535)) {
		$error = "Wrong port";
	}
	else if (strcmp($alg,"")==0) {
		$error = "Wrong algorithm";
	}
	else if (strcmp($key,"")==0) {
		$error = "Wrong key";
	}
	else if (strcmp($user,"")==0) {
		$error = "Wrong user account";
	}
	else if (strcmp($userId,"")==0) {
		$error = "Cannot get user's id";
	}
	else if ($timeout==0||strcmp($timeout,"")==0) {
		$error = "Bad timeout for the account";
	}
	else {
		$lm=new LdapManager($LDAP_SERVER_NAME,$LDAP_SERVER_PORT);
		$lm->setOption(LDAP_OPT_PROTOCOL_VERSION,3);
		$lm->connectLdap($LDAP_USER,$LDAP_PASS);
		
		$idnc = uuid();
		$entry = new LdapEntry("idnc=".$idnc.",".$LDAP_DN_BASE);
		$entry->addAttribute("objectClass","person");
		$entry->addAttribute("objectClass","ldapPublicKey");
		$entry->addAttribute("objectClass","schacUserEntitlements");
		$entry->addAttribute("objectClass","irisObject");
		$entry->addAttribute("objectClass","schacEntryMetadata");
		$entry->addAttribute("idnc",$idnc);
		$entry->addAttribute("uid",$userId);
		$entry->addAttribute("cn",$userId);
		$entry->addAttribute("sn",$userId);
		$entry->addAttribute("sshPublicKey",$alg." ".$key);
		$entry->addAttribute("schacUserStatus","urn:mace:terena.org:schac:userStatus:es:rediris.es:sshaccess:".urlencode($server)."+user=".$user);
		$entry->addAttribute("schacExpiryDate",date("YmdHis",$expiry)."Z");

		$objClassList=$entry->getObjectClassList()->getValues();
		$attrList=$entry->getAttributeValues();
        if ($lm->addEntry($entry->getDN(),$objClassList,$attrList)==false) {
			$error = "Error adding key";
		}
	}
}

header('Content-type: text/xml');
echo "<Response>";
$error = "";
if (array_key_exists("oper",$_REQUEST)) {
	$oper = $_REQUEST["oper"];
	if (strcmp($oper,"addkey")==0) {
		if (array_key_exists("server",$_REQUEST)&&array_key_exists("key",$_REQUEST)&&
			array_key_exists("alg",$_REQUEST)&&array_key_exists("user",$_REQUEST)) {
			$server = $_REQUEST["server"];
			$alg = $_REQUEST["alg"];
			$key = $_REQUEST["key"];
			$user = $_REQUEST["user"];
			addKey($server,$alg,$key,$user,&$error);
		}
		else {
			$error = "Wrong parameters";
		}
	}
	else if (strcmp($oper,"checkkey")==0) {
		if (array_key_exists("server",$_REQUEST)&&array_key_exists("user",$_REQUEST)) {
			$server = $_REQUEST["server"];
			$user = $_REQUEST["user"];
			checkKey($server,$user,&$error);
		}
		else {
			$error = "Wrong parameters";
		}
	}
	else if (strcmp($oper,"checkws")==0) {
		if (array_key_exists("server",$_REQUEST)) {
			$server = $_REQUEST["server"];
			$ws = "";
			if (array_key_exists("ws", $_REQUEST)) {
				$ws = $_REQUEST["ws"];
			}
			checkWS($server,$ws,&$error);
		}
		else {
			$error = "Wrong parameters";
		}
	}
	else if (strcmp($oper,"cancelkey")==0) {
		if (array_key_exists("server",$_REQUEST)&&array_key_exists("user",$_REQUEST)) {
			$server = $_REQUEST["server"];
			$user = $_REQUEST["user"];
			cancelKey($server,$user,&$error);
		}
		else {
			$error = "Wrong parameters";
		}
	}
	else if (strcmp($oper,"updatekey")==0) {
		if (array_key_exists("server",$_REQUEST)&&array_key_exists("user",$_REQUEST)) {
			$server = $_REQUEST["server"];
			$user = $_REQUEST["user"];
			updateKey($server,$user,&$error);
		}
		else {
			$error = "Wrong parameters";
		}
	}
	else {
		$error = "Bad operation";
	}
}
else {
	$error = "Request not valid";
}

if (strcmp($error,"")!=0) {
	echo '<OperationInfo><Status status="error" msg="'.$error.'"/></OperationInfo>';
}
else {
	echo "<OperationInfo><Status status='success'/></OperationInfo>";
}
echo "</Response>";
?>